@@ -5,9 +5,27 @@ set -o nounset
 set -o pipefail
 
 source ./scripts/test_lib.sh
+source ./scripts/release_mod.sh
 
 DRY_RUN=${DRY_RUN:-true}
 
+# Following preparation steps help with the release process: 
+
+# If you use password-protected gpg key, make sure the password is managed
+# by agent: 
+#
+# % gpg-connect-agent reloadagent /bye
+# % gpg -s --default-key [git-email]@google.com -o /dev/null -s /dev/null
+#
+# Refresh your google credentials: 
+#  % gcloud auth login
+# or
+#  % gcloud auth activate-service-account --key-file=gcp-key-etcd-development.json
+#
+# Make sure gcloud-docker plugin is configured: 
+#  % gcloud auth configure-docker
+
+
 help() {
   echo "$(basename "$0") [version]"
   echo "Release etcd using the same approach as the etcd-release-runbook (https://goo.gl/Gxwysq)"
@@ -19,18 +37,20 @@ help() {
   echo "         emails. These steps must be performed manually AFTER running this tool."
   echo ""
   echo "  args:"
-  echo "    version: version of etcd to release, e.g. '3.2.18'"
+  echo "    version: version of etcd to release, e.g. 'v3.2.18'"
   echo "  flags:"
   echo "    --no-upload: skip gs://etcd binary artifact uploads."
   echo "    --no-docker-push: skip docker image pushes."
   echo "    --in-place: build binaries using current branch."
   echo ""
+  echo "One can perform a (dry-run) test release from any (uncommitted) branch using:"
+  echo "  DRY_RUN=true REPOSITORY=\`pwd\` BRANCH='local-branch-name' ./scripts/release 3.5.0-foobar.2"
 }
 
 main() {
   VERSION=$1
   if [[ ! "${VERSION}" =~ [0-9]+.[0-9]+.[0-9]+ ]]; then
-    echo "Expected 'version' param of the form '<major-version>.<minor-version>.<patch-version>' but got '${VERSION}'"
+    log_error "Expected 'version' param of the form '<major-version>.<minor-version>.<patch-version>' but got '${VERSION}'"
     exit 1
   fi
   RELEASE_VERSION="v${VERSION}"
@@ -41,7 +61,7 @@ main() {
       REPOSITORY=$(pwd)
       BRANCH=$(git rev-parse --abbrev-ref HEAD)
   else
-      REPOSITORY=${REPOSITORY:-"https://github.com/etcd-io/etcd.git"}
+      REPOSITORY=${REPOSITORY:-"git@github.com:etcd-io/etcd.git"}
       BRANCH=${BRANCH:-"release-${MINOR_VERSION}"}
   fi
 
@@ -49,7 +69,11 @@ main() {
   log_callout "RELEASE_VERSION=${RELEASE_VERSION}"
   log_callout "MINOR_VERSION=${MINOR_VERSION}"
   log_callout "BRANCH=${BRANCH}"
+  log_callout "REPOSITORY=${REPOSITORY}"
   log_callout ""
+  
+  # Required to enable 'docker manifest ...'
+  export DOCKER_CLI_EXPERIMENTAL=enabled
 
   if ! command -v docker >/dev/null; then
     log_error "cannot find docker"
@@ -65,23 +89,28 @@ main() {
   if [ ! -d "${reldir}/etcd" ] && [ "${IN_PLACE}" == 0 ]; then
     mkdir -p "${reldir}"
     cd "${reldir}"
-    git clone "${REPOSITORY}" --branch "${BRANCH}"
-    cd "${reldir}/etcd" || exit 2
-    git checkout "${BRANCH}" || exit 2
-    git pull origin
+    run git clone "${REPOSITORY}" --branch "${BRANCH}" --depth 1
+    run cd "${reldir}/etcd" || exit 2
+    run git checkout "${BRANCH}" || exit 2
+    run git pull origin
+
+    git_assert_branch_in_sync || exit 2
   fi
-  
+
+  # mark local directory as root for test_lib scripts executions
+  set_root_dir
+
   # If a release version tag already exists, use it.
-  log_callout "Checking tag: ${RELEASE_VERSION}"
   local remote_tag_exists
-  remote_tag_exists=$(git ls-remote origin "refs/tags/${RELEASE_VERSION}" | grep -c "${RELEASE_VERSION}" || true)
+  remote_tag_exists=$(run git ls-remote origin "refs/tags/${RELEASE_VERSION}" | grep -c "${RELEASE_VERSION}" || true)
+
   if [ "${remote_tag_exists}" -gt 0 ]; then
     log_callout "Release version tag exists on remote. Checking out refs/tags/${RELEASE_VERSION}"
     git checkout -q "tags/${RELEASE_VERSION}"
   fi
 
   # Check go version.
-  log_callout "Checking go version"
+  log_callout "Check go version"
   local go_version current_go_version
   go_version="go$(cat .go-version)"
   current_go_version=$(go version | awk '{ print $3 }')
@@ -95,19 +124,23 @@ main() {
   if [ "${remote_tag_exists}" -eq 0 ]; then
     # Bump version/version.go to release version.
     local source_version
-    source_version=$(grep -E "\s+Version\s*=" version/version.go | sed -e "s/.*\"\(.*\)\".*/\1/g")
+    source_version=$(grep -E "\s+Version\s*=" api/version/version.go | sed -e "s/.*\"\(.*\)\".*/\1/g")
     if [[ "${source_version}" != "${VERSION}" ]]; then
       source_minor_version=$(echo "${source_version}" | cut -d. -f 1-2)
       if [[ "${source_minor_version}" != "${MINOR_VERSION}" ]]; then
-        log_error "Wrong etcd minor version in version/version.go. Expected ${MINOR_VERSION} but got ${source_minor_version}. Aborting."
+        log_error "Wrong etcd minor version in api/version/version.go. Expected ${MINOR_VERSION} but got ${source_minor_version}. Aborting."
         exit 1
       fi
-      echo "Updating version from ${source_version} to ${VERSION} in version/version.go"
-      sed -i "s/${source_version}/${VERSION}/g" version/version.go
+      log_callout "Updating modules definitions"
+      TARGET_VERSION="v${VERSION}" update_versions_cmd
+
+      log_callout "Updating version from ${source_version} to ${VERSION} in api/version/version.go"
+      sed -i "s/${source_version}/${VERSION}/g" api/version/version.go
     fi
 
+
     log_callout "Building etcd and checking --version output"
-    ./build
+    run ./build.sh
     local etcd_version
     etcd_version=$(bin/etcd --version | grep "etcd Version" | awk '{ print $3 }')
     if [[ "${etcd_version}" != "${VERSION}" ]]; then
@@ -116,18 +149,20 @@ main() {
     fi
 
     if [[ -n $(git status -s) ]]; then
-      log_callout "Committing version/version.go update."
-      git add version/version.go
-      git diff --staged | cat
-      git commit --signoff -m "version: bump up to ${VERSION}"
-      git diff --staged | cat
+      log_callout "Committing mods & api/version/version.go update."
+      run git add api/version/version.go
+      # shellcheck disable=SC2038,SC2046,SC2185
+      run git add $(find -name go.mod ! -path './release/*'| xargs)
+      run git diff --staged | cat
+      run git commit -m "version: bump up to ${VERSION}"
+      run git diff --staged | cat
     fi
 
     # Push the version change if it's not already been pushed.
-    if [ "$DRY_RUN" != "true" ] && [ "$(git rev-list --count "origin/${BRANCH}..${BRANCH}")" -gt 0 ]; then
-      read -p "Push version bump up to ${VERSION} to github.com/etcd-io/etcd [y/N]? " -r confirm
+    if [ "${DRY_RUN}" != "true" ] && [ "$(git rev-list --count "origin/${BRANCH}..${BRANCH}")" -gt 0 ]; then
+      read -p "Push version bump up to ${VERSION} to '$(git remote get-url origin)' [y/N]? " -r confirm
       [[ "${confirm,,}" == "y" ]] || exit 1
-      git push
+      maybe_run git push
     fi
 
     # Tag release.
@@ -135,21 +170,7 @@ main() {
       log_callout "Skipping tag step. git tag ${RELEASE_VERSION} already exists."
     else
       log_callout "Tagging release..."
-      gitemail=$(git config --get user.email)
-      KEYID=$(gpg --list-keys --with-colons "${gitemail}" | awk -F: '/^pub:/ { print $5 }')
-      if [[ -z "${KEYID}" ]]; then
-        log_error "Failed to load gpg key. Is gpg set up correctly for etcd releases?"
-        exit 1
-      fi
-      git tag --local-user "${KEYID}" --sign "${RELEASE_VERSION}" --message "${RELEASE_VERSION}"
-    fi
-
-    # Verify the latest commit has the version tag
-    # shellcheck disable=SC2155
-    local tag="$(git describe --exact-match HEAD)"
-    if [ "${tag}" != "${RELEASE_VERSION}" ]; then
-      log_error "Error: Expected HEAD to be tagged with ${RELEASE_VERSION}, but 'git describe --exact-match HEAD' reported: ${tag}"
-      exit 1
+      REMOTE_REPO="origin" push_mod_tags_cmd
     fi
 
     if [ "${IN_PLACE}" == 0 ]; then
@@ -185,7 +206,6 @@ main() {
     log_error "Error: Expected clean working tree, but 'git diff --stat' reported: ${diff}"
     exit 1
   fi
-  
 
   # Build release.
   # TODO: check the release directory for all required build artifacts.
@@ -193,14 +213,13 @@ main() {
     log_warning "Skipping release build step. /release directory already exists."
   else
     log_callout "Building release..."
-    # Check for old and new names of the release build script.
-    # TODO: Move the release script into this on as a function?
     REPOSITORY=$(pwd) ./scripts/build-release.sh "${RELEASE_VERSION}"
   fi
 
   # Sanity checks.
   "./release/etcd-${RELEASE_VERSION}-$(go env GOOS)-amd64/etcd" --version | grep -q "etcd Version: ${VERSION}" || true
   "./release/etcd-${RELEASE_VERSION}-$(go env GOOS)-amd64/etcdctl" version | grep -q "etcdctl version: ${VERSION}" || true
+  "./release/etcd-${RELEASE_VERSION}-$(go env GOOS)-amd64/etcdutl" version | grep -q "etcdutl version: ${VERSION}" || true
 
   # Generate SHA256SUMS
   log_callout "Generating sha256sums of release artifacts."
@@ -216,40 +235,51 @@ main() {
   fi
 
   # Upload artifacts.
-  if [ "$DRY_RUN" = "true" ] || [ "${NO_UPLOAD}" == 1 ]; then
-    log_callout "Skipping artifact upload to gs://etcd. --no-upload flat is set or DRY_RUN is true."
+  if [ "${DRY_RUN}" == "true" ] || [ "${NO_UPLOAD}" == 1 ]; then
+    log_callout "Skipping artifact upload to gs://etcd. --no-upload flat is set."
   else
     read -p "Upload etcd ${RELEASE_VERSION} release artifacts to gs://etcd [y/N]? " -r confirm
     [[ "${confirm,,}" == "y" ]] || exit 1
-    gsutil -m cp ./release/SHA256SUMS "gs://etcd/${RELEASE_VERSION}/"
-    gsutil -m cp ./release/*.zip "gs://etcd/${RELEASE_VERSION}/"
-    gsutil -m cp ./release/*.tar.gz "gs://etcd/${RELEASE_VERSION}/"
-    gsutil -m acl ch -u allUsers:R -r "gs://etcd/${RELEASE_VERSION}/"
+    maybe_run gsutil -m cp ./release/SHA256SUMS "gs://etcd/${RELEASE_VERSION}/"
+    maybe_run gsutil -m cp ./release/*.zip "gs://etcd/${RELEASE_VERSION}/"
+    maybe_run gsutil -m cp ./release/*.tar.gz "gs://etcd/${RELEASE_VERSION}/"
+    maybe_run gsutil -m acl ch -u allUsers:R -r "gs://etcd/${RELEASE_VERSION}/"
   fi
 
   # Push images.
-  if [ "$DRY_RUN" = "true" ] || [ "${NO_DOCKER_PUSH}" == 1 ]; then
-    log_callout "Skipping docker push. --no-docker-push flat is set or DRY_RUN is true."
+  if [ "${DRY_RUN}" == "true" ] || [ "${NO_DOCKER_PUSH}" == 1 ]; then
+    log_callout "Skipping docker push. --no-docker-push flat is set."
   else
     read -p "Publish etcd ${RELEASE_VERSION} docker images to quay.io [y/N]? " -r confirm
     [[ "${confirm,,}" == "y" ]] || exit 1
-
-    echo "Pushing container images to quay.io ${RELEASE_VERSION}"
-    docker push "quay.io/coreos/etcd:${RELEASE_VERSION}"
-
-    echo "Pushing container images to gcr.io ${RELEASE_VERSION}"
-    docker push "gcr.io/etcd-development/etcd:${RELEASE_VERSION}"
-
-    for TARGET_ARCH in "-arm64" "-ppc64le"; do
-      log_callout "Pushing container images to quay.io ${RELEASE_VERSION}${TARGET_ARCH}"
-      docker push "quay.io/coreos/etcd:${RELEASE_VERSION}${TARGET_ARCH}"
-
-      log_callout "Pushing container images to gcr.io ${RELEASE_VERSION}${TARGET_ARCH}"
-      docker push "gcr.io/etcd-development/etcd:${RELEASE_VERSION}${TARGET_ARCH}"
+    # shellcheck disable=SC2034
+    for i in {1..5}; do
+      docker login quay.io && break
+      log_warning "login failed, retrying"
     done
 
-    log_callout "Setting permissions using gsutil..."
-    gsutil -m acl ch -u allUsers:R -r gs://artifacts.etcd-development.appspot.com
+    for TARGET_ARCH in "amd64" "arm64" "ppc64le" "s390x"; do
+      log_callout "Pushing container images to quay.io ${RELEASE_VERSION}-${TARGET_ARCH}"
+      maybe_run docker push "quay.io/coreos/etcd:${RELEASE_VERSION}-${TARGET_ARCH}"
+      log_callout "Pushing container images to gcr.io ${RELEASE_VERSION}-${TARGET_ARCH}"
+      maybe_run docker push "gcr.io/etcd-development/etcd:${RELEASE_VERSION}-${TARGET_ARCH}"
+    done
+
+    log_callout "Creating manifest-list (multi-image)..."
+
+    for TARGET_ARCH in "amd64" "arm64" "ppc64le" "s390x"; do
+      maybe_run docker manifest create --amend "quay.io/coreos/etcd:${RELEASE_VERSION}" "quay.io/coreos/etcd:${RELEASE_VERSION}-${TARGET_ARCH}"
+      maybe_run docker manifest annotate "quay.io/coreos/etcd:${RELEASE_VERSION}" "quay.io/coreos/etcd:${RELEASE_VERSION}-${TARGET_ARCH}" --arch "${TARGET_ARCH}"
+
+      maybe_run docker manifest create --amend "gcr.io/etcd-development/etcd:${RELEASE_VERSION}" "gcr.io/etcd-development/etcd:${RELEASE_VERSION}-${TARGET_ARCH}"
+      maybe_run docker manifest annotate "gcr.io/etcd-development/etcd:${RELEASE_VERSION}" "gcr.io/etcd-development/etcd:${RELEASE_VERSION}-${TARGET_ARCH}" --arch "${TARGET_ARCH}"
+    done
+
+    log_callout "Pushing container manifest list to quay.io ${RELEASE_VERSION}"
+    maybe_run docker manifest push "quay.io/coreos/etcd:${RELEASE_VERSION}"
+
+    log_callout "Pushing container manifest list to gcr.io ${RELEASE_VERSION}"
+    maybe_run docker manifest push "gcr.io/etcd-development/etcd:${RELEASE_VERSION}"
   fi
 
   ### Release validation
@@ -257,6 +287,9 @@ main() {
 
   # Check image versions
   for IMAGE in "quay.io/coreos/etcd:${RELEASE_VERSION}" "gcr.io/etcd-development/etcd:${RELEASE_VERSION}"; do
+    if [ "${DRY_RUN}" == "true" ] || [ "${NO_DOCKER_PUSH}" == 1 ]; then
+      IMAGE="${IMAGE}-amd64"
+    fi
     # shellcheck disable=SC2155
     local image_version=$(docker run --rm "${IMAGE}" etcd --version | grep "etcd Version" | awk -F: '{print $2}' | tr -d '[:space:]')
     if [ "${image_version}" != "${VERSION}" ]; then
